/*
  Model-View-Controller demo program.
  Copyright (C) 2006 by Maria Litvin and Gary Litvin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details:
    http://www.gnu.org/licenses/gpl.html.
*/

import javax.swing.JFrame;

// This program demonstrates the MVC design pattern.
public class MVCDemo
{
  public static void main(String[] args)
  {
    Car car = new Car();  // This is a "model"

    JFrame brake = new JFrame("Brake");
    brake.setBounds(300, 150, 140, 100);
    brake.getContentPane().add(new BrakePedal(car)); // "Controller" 1
    brake.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    brake.setResizable(false);
    brake.setVisible(true);

    JFrame gas = new JFrame("Gas");
    gas.setBounds(450, 150, 140, 100);
    gas.getContentPane().add(new GasPedal(car)); // "Controller" 2
    gas.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    gas.setResizable(false);
    gas.setVisible(true);

    Clock clock = new Clock(car, 30); // "Controller" 3; moves the car
                                      //   every 30 ms

    RoadsideDisplay roadsideDisplay = new RoadsideDisplay(); // "View" 1
    car.addObserver(roadsideDisplay);
    JFrame displayFrame = new JFrame("SPEED LIMIT 30");
    displayFrame.setBounds(700, 360, 200, 140);
    displayFrame.getContentPane().add(roadsideDisplay);
    displayFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    displayFrame.setResizable(false);
    displayFrame.setVisible(true);

    Speedometer speedometer = new Speedometer(); // "View" 2
    car.addObserver(speedometer);
    JFrame dashBoard = new JFrame();
    dashBoard.setBounds(395, 20, 120, 100);
    dashBoard.getContentPane().add(speedometer);
    dashBoard.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    dashBoard.setResizable(false);
    dashBoard.setVisible(true);

    Road road = new Road(); // "View" 3
    car.addObserver(road);
    JFrame roadFrame = new JFrame();
    roadFrame.setBounds(100, 500, 1000, 180);
    roadFrame.getContentPane().add(road);
    roadFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    roadFrame.setResizable(false);
    roadFrame.setVisible(true);

    car.addObserver(new TiresSqueal()); // "View" 4

    clock.start();
  }
}
