/*
  Model-View-Controller demo program.
  Copyright (C) 2006 by Maria Litvin and Gary Litvin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details:
    http://www.gnu.org/licenses/gpl.html.
*/

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import javax.swing.JPanel;
import java.util.Observer;
import java.util.Observable;

// This class represents a speedometer gauge.
public class Speedometer extends JPanel
    implements Observer
{
  private Car car;

  // Called when the model (a Car) notifies its observers that
  // it has moved.
  public void update(Observable model, Object arg)
  {
    car = (Car)model;
    repaint();
  }

  // Repaints this panel.
  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    int x0 = getWidth() / 2;
    int y0 = 3 * getHeight() / 4;
    int r = getWidth() / 4;

    // Draw the gauge:
    g.setColor(Color.BLACK);
    g.drawArc(x0 - r - 3, y0 - r - 3, 2*r + 8, 2*r + 8, 0, 180);
    g.setColor(Color.DARK_GRAY);
    g.fillArc(x0 - r, y0 - r, 2*r+2, 2*r+2, 0, 180);


    // Draw the numbers:
    double radians;
    int x1, y1;

    g.setColor(Color.BLACK);
    g.setFont(new Font("Sanserif", Font.PLAIN, 8));

    for (int k = 0; k <= 4; k++)
    {
      radians = Math.PI *(1.0 - 0.25*k);
      x1 = x0 + (int)(Math.round((r + 8) * Math.cos(radians))) - 5 + k;
      y1 = y0 - (int)(Math.round((r + 8) * Math.sin(radians)));
      g.drawString(String.valueOf(30 * k), x1, y1);
    }

    // Draw the pointer:
    g.setColor(Color.RED);
    g.fillArc(x0 - 3, y0 - 3, 8, 8, 0, 180);

    double speed = 0;
    if (car != null)
      speed = Math.abs(car.getSpeed()) *.0006214 * 3600;

    radians = Math.max(Math.PI * (1.0 - speed / 120), 0.0);
    x1 = x0 + (int)(Math.round((r - 5) * Math.cos(radians)));
    y1 = y0 - (int)(Math.round((r - 5) * Math.sin(radians)));
    g.drawLine(x0, y0, x1, y1);
  }
}
