/*
  Model-View-Controller demo program.
  Copyright (C) 2006 by Maria Litvin and Gary Litvin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details:
    http://www.gnu.org/licenses/gpl.html.
*/

import java.util.Observer;
import java.util.Observable;

// Tires squeal when the brakes are locked.
// Runs in a separate thread.
public class TiresSqueal extends Thread
    implements Observer
{
  private enum State {RUNNING, BRAKING, STOPPED};

  private EasySound squealingTiresSound = new EasySound("squealingtires.wav");
  private boolean playing = false;
  private State carState = State.RUNNING;

  // constructs this thread and starts it.
  public TiresSqueal()
  {
    start();
  }

  // Called when the model (a Car) notifies its observers that
  // it has moved.
  public void update(Observable model, Object arg)
  {
    boolean brakesLocked = ((Car)model).roughStop();
    if (brakesLocked && carState == State.RUNNING)
      carState = State.BRAKING;
    else if (!brakesLocked)
      carState = State.RUNNING;
  }

  // This thread is always running
  public void run()
  {
    try
    {
      while (true)
      {
        // Play the sound only once while the car is braking.
        if (carState == State.BRAKING)
        {
          squealingTiresSound.play();
          carState = State.STOPPED;
        }
        sleep(500);
      }
    }
    catch (InterruptedException ex)
    {
    }
  }
}
